﻿// 2011, Omer Faruk ZORLU
// http://www.omerfarukzorlu.com
using System.Data;
using System.Linq;
using System.Text;
using Microsoft.ApplicationBlocks.Data;
using OMR.CommerceSite.DataAccess;
using OMR.DataSync.Core.Helpers;
using OMR.TableDataSync.Console.Configuration;
using OMR.TableDataSync.Console.Helpers;
using OMR.TableDataSync.Core.DataAccess.Core;
using OMR.TableDataSync.Core.Domain;
using System.Data.SqlClient;

namespace OMR.TableDataSync.Core.DataAccess
{
    public class TransferBLL : BaseBLL
    {
        public TransferBLL(string connectionString)
            : base(connectionString)
        { }

        private static string GenerateSelectSql(TableConfiguration tableConfiguration, TableHash tableHash, TableHashTargets target)
        {
            ValidationHelper.AreNotNull(tableConfiguration, tableHash);

            var primaryKeys = tableConfiguration.ColumnConfigurations.Where(k => k.IsPrimaryKey == true).ToList();
            var allColumns = tableConfiguration.ColumnConfigurations.Select(k => k.Name); // TODO: isimini seçmeden

            // TODO: Paging
            StringBuilder sb = new StringBuilder();
            sb.AppendFormat("SELECT {0} FROM {1} NOLOCK ", string.Join(", ", allColumns), tableConfiguration.Name);

            GenerateWhereStatement(tableHash, target, sb);

            return sb.ToString();
        }

        private static string GenerateInsertSql(TableConfiguration tableConfiguration)
        {
            ValidationHelper.AreNotNull(tableConfiguration);

            var allColumns = tableConfiguration.ColumnConfigurations.Select(k => k.Name);

            var strSql = string.Format("SET IDENTITY_INSERT {0} ON; INSERT INTO {0}({1}) VALUES(@{2}); SET IDENTITY_INSERT {0} OFF; ",
                                tableConfiguration.Name,
                                string.Join(",", allColumns),
                                string.Join(", @", allColumns)
                            );

            return strSql;
        }

        private static string GenerateUpdateSql(TableConfiguration tableConfiguration)
        {
            ValidationHelper.AreNotNull(tableConfiguration);

            var primaryKeys = tableConfiguration.ColumnConfigurations.Where(k => k.IsPrimaryKey == true).ToArray();
            var updateColumns = tableConfiguration.ColumnConfigurations.Where(k => k.IsPrimaryKey == false && k.IgnoreOnUpdate == false).Select(k => k.Name).ToArray();

            StringBuilder sb = new StringBuilder();
            sb.AppendFormat("UPDATE {0} SET ", tableConfiguration.Name);
            for (int i = 0; i < updateColumns.Length; i++)
            {
                sb.AppendFormat("{0} = @{0} ", updateColumns[i]);
                if (i != updateColumns.Length - 1)
                    sb.Append(",");
            }

            sb.Append("WHERE ");
            for (int i = 0; i < primaryKeys.Length; i++)
            {
                sb.AppendFormat("{0} = @{0} ", primaryKeys[i]);
                if (i != primaryKeys.Length - 1)
                    sb.Append("AND ");
            }

            return sb.ToString();
        }

        private static string GenerateDeleteSql(TableConfiguration tableConfiguration, TableHash tableHash)
        {
            ValidationHelper.AreNotNull(tableConfiguration, tableHash);

            var allColumns = tableConfiguration.ColumnConfigurations.Select(k => k.Name); // TODO: isimini seçmeden

            // TODO: Paging
            StringBuilder sb = new StringBuilder();
            sb.AppendFormat("DELETE FROM {0} ", tableConfiguration.Name);

            GenerateWhereStatement(tableHash, TableHashTargets.WillBeDeleted, sb);

            return sb.ToString();
        }

        private static void GenerateWhereStatement(TableHash tableHash, TableHashTargets target, StringBuilder sb)
        {
            ValidationHelper.AreNotNull(tableHash, sb);

            var targetRowHashes = tableHash.RowHashes.Where(k => k.Target == target);

            for (int i = 0; i < tableHash.PrimaryKeyNames.Count(); i++)
            {
                if (0 == i)
                    sb.Append("WHERE ");
                else
                    sb.Append("AND ");

                sb.AppendFormat("{0} IN (", tableHash.PrimaryKeyNames[i]);

                // Append primary key values
                sb.AppendFormat("'{0}'", string.Join("', '", targetRowHashes.Select(k => k.PrimaryKeyValues[i])));

                sb.Append(") ");
            }

            sb.Append(";");
        }

        internal void InsertRows(TableConfiguration tableConfiguration, TableHash tableHash, string destinationConnectionString)
        {
            ValidationHelper.AreNotNull(tableConfiguration, tableHash, destinationConnectionString);
            TraceHelper.Write("{0} Preparing for the process of inserting\n", 6, tableConfiguration.Name);

            string strSql = GenerateSelectSql(tableConfiguration, tableHash, TableHashTargets.WillBeInserted);

            int executeCount = 0;

            if (base.DbConnection.State == ConnectionState.Closed)
                base.DbConnection.Open();

            using (var sdr = SqlHelper.ExecuteReader(base.DbConnection, CommandType.Text, strSql))
            {
                while (sdr.Read())
                {
                    SqlParameterHelper sph = new SqlParameterHelper(sdr.FieldCount);
                    for (int i = 0; i < sdr.FieldCount; i++)
                    {
                        sph.Add("@" + sdr.GetName(i), sdr[i]);
                    }

                    // TODO: Check and report
                    SqlHelper.ExecuteNonQuery(destinationConnectionString, CommandType.Text, GenerateInsertSql(tableConfiguration), sph.ToArray());

                    if (executeCount % 100 == 0)
                        TraceHelper.Write("{0} proceed\n", 6, executeCount);

                    executeCount++;
                }
            }

            TraceHelper.Write("{0} Insert operation is complated\n", 6, tableConfiguration.Name);
        }

        internal void UpdateRows(TableConfiguration tableConfiguration, TableHash tableHash, string destinationConnectionString)
        {
            ValidationHelper.AreNotNull(tableConfiguration, tableHash, destinationConnectionString);
            TraceHelper.Write("{0} Preparing for the process of updating\n", 6, tableConfiguration.Name);

            string strSql = GenerateSelectSql(tableConfiguration, tableHash, TableHashTargets.WillBeUpdated);

            var sdr = SqlHelper.ExecuteReader(base.DbConnection, CommandType.Text, strSql);

            int executeCount = 0;

            while (sdr.Read())
            {
                SqlParameterHelper sph = new SqlParameterHelper(sdr.FieldCount);
                for (int i = 0; i < sdr.FieldCount; i++)
                {
                    sph.Add("@" + sdr.GetName(i), sdr[i]);
                }

                strSql = GenerateUpdateSql(tableConfiguration);

                SqlHelper.ExecuteNonQuery(destinationConnectionString, CommandType.Text, strSql, sph.ToArray());

                if (executeCount % 100 == 0)
                    TraceHelper.Write("{0} proceed\n", 6, executeCount);

                executeCount++;
            }

            sdr.Close();

            TraceHelper.Write("{0} Update operation is complated\n", 6, tableConfiguration.Name);
        }

        internal void DeleteRows(TableConfiguration tableConfiguration, TableHash tableHash, string destinationConnectionString)
        {
            ValidationHelper.AreNotNull(tableConfiguration, tableHash, destinationConnectionString);
            TraceHelper.Write("{0} Preparing for the process of inserting\n", 6, tableConfiguration.Name);

            string strSql = GenerateDeleteSql(tableConfiguration, tableHash);

            SqlHelper.ExecuteNonQuery(destinationConnectionString, CommandType.Text, strSql);

            TraceHelper.Write("{0} Delete operation is complated\n", 6, tableConfiguration.Name);
        }
    }
}
